<?php
namespace Drupal\paragraphs_table\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\field\FieldConfigInterface;
use Symfony\Component\HttpFoundation\JsonResponse;

/**
 * Returns responses for paragraphs item routes.
 */
class AjaxController extends ControllerBase {
  protected $renderBuild;
  protected $fieldsDefinition;
  public function json($field_name, $host_type, $host_id) {
    $message = [];
    $entity = $this->entityTypeManager()->getStorage($host_type)->load($host_id);
    if (empty($entity)) {
      $message[] = $this->t("%type %id doesn't exist", [
        "%type" => $host_type,
        "%id" => $host_id,
      ]);
    }
    $paragraph_field = $entity->hasField($field_name) ? $entity->$field_name : FALSE;
    if (empty($paragraph_field)) {
      $message[] = $this->t("%field that does not exist on entity type %type", [
        "%field" => $field_name,
        "%type" => $host_type,
      ]);
    }

    if($paragraph_field && $paragraph_field_items = $paragraph_field->referencedEntities()){
      $components = $this->getComponents($paragraph_field);
      $typeLabel = \Drupal::request()->get('type');
      if(empty($typeLabel)){
        $typeLabel = 'data';
      }
      return new JsonResponse([
        $typeLabel => $this->getResults($paragraph_field_items,$components),
      ]);
    }else{
      $message[] = $this->t("%field is not paragraphs", [
        "%field" => $field_name,
      ]);
    }
    return new JsonResponse([
      'error' => $message,
    ]);
  }
  protected function getComponents($paragraph_field, $view_mode = 'default'){
    $field_definition = $paragraph_field->getFieldDefinition();
    $targetBundle = array_key_first($field_definition->getSetting("handler_settings")["target_bundles"]);
    $targetType = $field_definition->getSetting('target_type');
    $fieldsDefinitions = $this->entityTypeManager()->getStorage($targetType)
      ->create(['type' => $targetBundle])->getFieldDefinitions();
    $repository = \Drupal::service('entity_display.repository');
    $viewDisplay = $repository->getViewDisplay($targetType, $targetBundle, $view_mode);
    $components = $viewDisplay->getComponents();
    uasort($components, 'Drupal\Component\Utility\SortArray::sortByWeightElement');
    foreach ($components as $field_name => $component) {
      if ($fieldsDefinitions[$field_name] instanceof FieldConfigInterface) {
        $this->fieldsDefinition[$field_name] = $fieldsDefinitions[$field_name];
      }
    }
    $storage = $this->entityTypeManager()->getStorage('entity_view_display');
    $this->renderBuild = $storage->load(implode('.', [$targetType, $targetBundle, $view_mode]));

    return $components;
  }
  public function getResults($entities,$components, $setting=[]) {
    $data = FALSE;
    foreach ($entities as $delta => $entitie) {
      $table_entity = $this->renderBuild->build($entitie);
      $objectData = new \stdClass();
      foreach ($components as $field_name => $field) {
        $table_entity[$field_name]['#label_display'] = 'hidden';
        $value = trim(strip_tags(render($table_entity[$field_name])));
        if (in_array($this->fieldsDefinition[$field_name]->getType(), [
          'integer',
          'list_integer',
          'number_integer',
        ])) {
          $value = (int) $value;
        }
        if (in_array($this->fieldsDefinition[$field_name]->getType(), ['boolean'])) {
          $list_value = $table_entity[$field_name]["#items"]->getValue();
          $value = (int) $list_value[0]['value'];
        }
        if (in_array($this->fieldsDefinition[$field_name]->getType(), [
          'decimal',
          'list_decimal',
          'number_decimal',
          'float',
          'list_float',
          'number_float',
        ])) {
          $value = (float) $value;
        }
        if (!empty($value)) {
          $notEmptyColumn[$field_name] = TRUE;
        }
        elseif (!empty($setting["empty_cell_value"])) {
          $value = $setting["empty_cell_value"];
        }

        $objectData->$field_name = $value;
      }
      $data[$delta] = $objectData;
    }
    return $data;
  }

  public function ajax($field_name, $host_type, $host_id) {
    $entity =  $this->entityTypeManager()->getStorage($host_type)->load($host_id);
    $paragraph_field = $entity->hasField($field_name) ? $entity->$field_name : FALSE;
    $setting = \Drupal::request()->get('setting');
    $components = $this->getComponents($paragraph_field, $setting['view_mode']);
    if($paragraph_field && $paragraph_field_items = $paragraph_field->referencedEntities()){
      return new JsonResponse($this->getTable($paragraph_field_items,$components,$setting));
    }
  }

  public function getTable($entities,$components, $setting=[]) {
    $data = FALSE;
    foreach ($entities as $delta => $entitie) {
      $table_entity = $this->renderBuild->build($entitie);
      foreach ($components as $field_name => $field) {
        $table_entity[$field_name]['#label_display'] = 'hidden';
        $value = trim(render($table_entity[$field_name]));
        if (!empty($value)) {
          $notEmptyColumn[$field_name] = TRUE;
        }
        elseif (!empty($setting["empty_cell_value"])) {
          $value = $setting["empty_cell_value"];
        }
        $data[$delta][] = $value;
      }
    }
    return $data;
  }
}
